@@ -1,4 +1,5 @@
 import logging
+import string
 
 import click
 
@@ -34,7 +35,27 @@ def plot(spec: str):
     """Plot a ScanSpec"""
     from scanspec.plot import plot_spec
 
-    for letter in "abcuvwxyz":
+    for letter in string.ascii_lowercase:
         locals()[letter] = letter
     eval_spec = eval(spec)
     plot_spec(eval_spec)
+
+
+@cli.command()
+@click.option("--cors", is_flag=True)
+@click.option(
+    "--port", default=8080, help="The port that the scanspec service will be hosted on."
+)
+def service(cors, port):
+    """Run up a GraphQL service"""
+    from scanspec.service import run_app
+
+    run_app(cors, port)
+
+
+@cli.command()
+def schema():
+    """Print schema"""
+    from scanspec.service import schema_text
+
+    click.echo(schema_text())
