@@ -1,8 +1,9 @@
-from dataclasses import dataclass, field, is_dataclass
+from dataclasses import dataclass, is_dataclass
 from typing import Iterator, List, Set
 
 import numpy as np
 from apischema import schema
+from typing_extensions import Annotated as A
 
 from .core import AxesPoints, Serializable, if_instance_do
 
@@ -16,7 +17,9 @@ __all__ = [
     "SymmetricDifferenceOf",
     "Range",
     "Rectangle",
+    "Polygon",
     "Circle",
+    "Ellipse",
     "find_regions",
 ]
 
@@ -85,10 +88,8 @@ def _merge_axis_sets(axis_sets: List[Set[str]]) -> Iterator[Set[str]]:
 class CombinationOf(Region):
     """Abstract baseclass for a combination of two regions, left and right"""
 
-    left: Region = field(metadata=schema(description="The left-hand Region to combine"))
-    right: Region = field(
-        metadata=schema(description="The right-hand Region to combine")
-    )
+    left: A[Region, schema(description="The left-hand Region to combine")]
+    right: A[Region, schema(description="The right-hand Region to combine")]
 
     def axis_sets(self) -> List[Set[str]]:
         axis_sets = list(
@@ -169,15 +170,9 @@ class Range(Region):
     array([False,  True,  True, False, False])
     """
 
-    axis: str = field(
-        metadata=schema(description="The name matching the axis to mask in spec")
-    )
-    min: float = field(
-        metadata=schema(description="The minimum inclusive value in the region")
-    )
-    max: float = field(
-        metadata=schema(description="The minimum inclusive value in the region")
-    )
+    axis: A[str, schema(description="The name matching the axis to mask in spec")]
+    min: A[float, schema(description="The minimum inclusive value in the region")]
+    max: A[float, schema(description="The minimum inclusive value in the region")]
 
     def axis_sets(self) -> List[Set[str]]:
         return [{self.axis}]
@@ -201,28 +196,15 @@ class Rectangle(Region):
         spec = grid & Rectangle("x", "y", 0, 1.1, 1.5, 2.1, 30)
     """
 
-    x_axis: str = field(
-        metadata=schema(description="The name matching the x axis of the spec")
-    )
-    y_axis: str = field(
-        metadata=schema(description="The name matching the y axis of the spec")
-    )
-    x_min: float = field(
-        metadata=schema(description="Minimum inclusive x value in the region")
-    )
-    y_min: float = field(
-        metadata=schema(description="Minimum inclusive y value in the region")
-    )
-    x_max: float = field(
-        metadata=schema(description="Maximum inclusive x value in the region")
-    )
-    y_max: float = field(
-        metadata=schema(description="Maximum inclusive y value in the region")
-    )
-    angle: float = field(
-        default=0.0,
-        metadata=schema(description="Clockwise rotation angle of the rectangle"),
-    )
+    x_axis: A[str, schema(description="The name matching the x axis of the spec")]
+    y_axis: A[str, schema(description="The name matching the y axis of the spec")]
+    x_min: A[float, schema(description="Minimum inclusive x value in the region")]
+    y_min: A[float, schema(description="Minimum inclusive y value in the region")]
+    x_max: A[float, schema(description="Maximum inclusive x value in the region")]
+    y_max: A[float, schema(description="Maximum inclusive y value in the region")]
+    angle: A[
+        float, schema(description="Clockwise rotation angle of the rectangle")
+    ] = 0.0
 
     def axis_sets(self) -> List[Set[str]]:
         return [{self.x_axis, self.y_axis}]
@@ -242,6 +224,51 @@ class Rectangle(Region):
         return mask_x & mask_y
 
 
+@dataclass
+class Polygon(Region):
+    """Mask contains points of axis within a rotated xy polygon
+
+    .. example_spec::
+
+        from scanspec.specs import Line
+        from scanspec.regions import Polygon
+
+        grid = Line("y", 3, 8, 10) * ~Line("x", 1 ,8, 10)
+        spec = grid & Polygon("x", "y", [1.0, 6.0, 8.0, 2.0], [4.0, 10.0, 6.0, 1.0])
+    """
+
+    x_axis: A[str, schema(description="The name matching the x axis of the spec")]
+    y_axis: A[str, schema(description="The name matching the y axis of the spec")]
+    x_verts: A[
+        List[float],
+        schema(description="The Nx1 x coordinates of the polygons vertices", min_len=3),
+    ]
+    y_verts: A[
+        List[float],
+        schema(description="The Nx1 y coordinates of the polygons vertices", min_len=3),
+    ]
+
+    def axis_sets(self) -> List[Set[str]]:
+        return [{self.x_axis, self.y_axis}]
+
+    def mask(self, points: AxesPoints) -> np.ndarray:
+        x = points[self.x_axis]
+        y = points[self.y_axis]
+        v1x, v1y = self.x_verts[-1], self.y_verts[-1]
+        mask = np.full(len(x), False, dtype=np.int8)
+        for v2x, v2y in zip(self.x_verts, self.y_verts):
+            # skip horizontal edges
+            if v2y != v1y:
+                vmask = np.full(len(x), False, dtype=np.int8)
+                vmask |= (y < v2y) & (y >= v1y)
+                vmask |= (y < v1y) & (y >= v2y)
+                t = (y - v1y) / (v2y - v1y)
+                vmask &= x < v1x + t * (v2x - v1x)
+                mask ^= vmask
+            v1x, v1y = v2x, v2y
+        return mask
+
+
 @dataclass
 class Circle(Region):
     """Mask contains points of axis within an xy circle of given radius
@@ -255,30 +282,66 @@ class Circle(Region):
         spec = grid & Circle("x", "y", 1, 2, 0.9)
     """
 
-    x_axis: str = field(
-        metadata=schema(description="The name matching the x axis of the spec")
-    )
-    y_axis: str = field(
-        metadata=schema(description="The name matching the x axis of the spec")
-    )
-    x_centre: float = field(
-        metadata=schema(description="Minimum inclusive x value in the region")
-    )
-    y_centre: float = field(
-        metadata=schema(description="Minimum inclusive y value in the region")
-    )
-    radius: float = field(metadata=schema(description="Radius of the circle"))
+    x_axis: A[str, schema(description="The name matching the x axis of the spec")]
+    y_axis: A[str, schema(description="The name matching the y axis of the spec")]
+    x_middle: A[float, schema(description="The central x point of the circle")]
+    y_middle: A[float, schema(description="The central y point of the circle")]
+    radius: A[float, schema(description="Radius of the circle", exc_min=0)]
 
     def axis_sets(self) -> List[Set[str]]:
         return [{self.x_axis, self.y_axis}]
 
     def mask(self, points: AxesPoints) -> np.ndarray:
-        x = points[self.x_axis] - self.x_centre
-        y = points[self.y_axis] - self.y_centre
+        x = points[self.x_axis] - self.x_middle
+        y = points[self.y_axis] - self.y_middle
         mask = x * x + y * y <= (self.radius * self.radius)
         return mask
 
 
+@dataclass
+class Ellipse(Region):
+    """Mask contains points of axis within an xy ellipse of given radius
+
+    .. example_spec::
+
+        from scanspec.specs import Line
+        from scanspec.regions import Ellipse
+
+        grid = Line("y", 3, 8, 10) * ~Line("x", 1 ,8, 10)
+        spec = grid & Ellipse("x", "y", 5, 5, 2, 3, 75)
+    """
+
+    x_axis: A[str, schema(description="The name matching the x axis of the spec")]
+    y_axis: A[str, schema(description="The name matching the y axis of the spec")]
+    x_middle: A[float, schema(description="The central x point of the ellipse")]
+    y_middle: A[float, schema(description="The central y point of the ellipse")]
+    x_radius: A[
+        float,
+        schema(description="The radius along the x axis of the ellipse", exc_min=0),
+    ]
+    y_radius: A[
+        float,
+        schema(description="The radius along the y axis of the ellipse", exc_min=0),
+    ]
+    angle: A[float, schema(description="The angle of the ellipse (degrees)")] = 0.0
+
+    def axis_sets(self) -> List[Set[str]]:
+        return [{self.x_axis, self.y_axis}]
+
+    def mask(self, points: AxesPoints) -> np.ndarray:
+        x = points[self.x_axis] - self.x_middle
+        y = points[self.y_axis] - self.y_middle
+        if self.angle != 0:
+            # Rotate src points by -angle
+            phi = np.radians(-self.angle)
+            tx = x * np.cos(phi) - y * np.sin(phi)
+            ty = x * np.sin(phi) + y * np.cos(phi)
+            x = tx
+            y = ty
+        mask = (x / self.x_radius) ** 2 + (y / self.y_radius) ** 2 <= 1
+        return mask
+
+
 def find_regions(obj) -> Iterator[Region]:
     """Recursively iterate over obj and its children, yielding any Region
     instances found"""
